package org.jboss.jtademo;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Serializable;

import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;

public class FileXAResource implements XAResource, Serializable {

	private static final long serialVersionUID = 1347040619772964325L;

	private static final String PREPARE_DIR = "xaDir/prepare";
	private static final String COMMIT_DIR = "xaDir/store";
		
	private String contents;
	private String filename;
	
	private int transactionTimeout = 0;

	public FileXAResource(String filename, String contents) {
		super();
		this.contents = contents;
		this.filename = filename;
	}

	public void commit(Xid xid, boolean onePhase) throws XAException {
		System.out.println("\tcommit xid=" + xid + ", onePhase=" + onePhase);
		if (onePhase) {
			try {
				PrintWriter pw = new PrintWriter(new FileWriter(new File(COMMIT_DIR, filename)));
				pw.println(contents);
				pw.close();
			}
			catch (IOException e) {
				System.out.println("\tCommit failed");
				throw new XAException(XAException.XA_RBINTEGRITY);
			}
		}
		else {
			File origFile = new File(PREPARE_DIR, filename);
			File destFile = new File(COMMIT_DIR, filename);
			System.out.println("\tMoving " + origFile + " -> " + destFile + " - " + origFile.renameTo(destFile));
		}
		System.out.println("Check contents of commit dir");
		try {
			Thread.sleep(5000);
		}
		catch (InterruptedException e) {
		}
		System.out.println("\tCommitted");
	}

	public void end(Xid xid, int flags) {
		System.out.println("\tend xid=" + xid + ", flags=" + Util.convertTMFlags(flags));
	}

	public void forget(Xid xid) {
		System.out.println("\tforget xid=" + xid);
	}

	public int getTransactionTimeout() {
		System.out.println("\tgetTransactionTimeout");
		return transactionTimeout;
	}

	public boolean isSameRM(XAResource xares) {
		System.out.println("\tisSame");
		return this == xares;
	}


	public int prepare(Xid xid) throws XAException {
		System.out.println("\tprepare xid=" + xid);
		try {
			PrintWriter pw = new PrintWriter(new FileWriter(new File(PREPARE_DIR, filename)));
			pw.println(contents);
			pw.close();
		}
		catch (IOException e) {
			throw new XAException(XAException.XA_RBINTEGRITY);
		}
		System.out.println("Check contents of prepare dir");
		try {
			System.in.read();
		}
		catch (IOException e) {
		}
		return XA_OK;
	}

	public Xid[] recover(int flag) {
		System.out.println("\trecover flag=" + Util.convertTMFlags(flag));
		return new Xid[0];
	}

	public void rollback(Xid xid) {
		System.out.println("\trollback xid=" + xid);
	}

	public boolean setTransactionTimeout(int seconds) {
		System.out.println("\tsetTransactionTimeout seconds=" + seconds);
		this.transactionTimeout = seconds;
		return true;
	}

	public void start(Xid xid, int flags) {
		System.out.println("\tstart xid=" + xid + ", flags=" + Util.convertTMFlags(flags));
	}

//	private void writeObject(ObjectOutputStream out) throws IOException {
//		System.out.println("\tStore object");
//	}
//
//	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
//		System.out.println("\tRecover object");
//	}

}
